@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017, 2018, 2019  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018, 2019, 2020  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -199,13 +199,10 @@ struct json_token_literal_part_t
 enum json_token_number_state_t
 {
     json_token_number_start_state,
-    json_token_number_integer_state,
-    json_token_number_integer2_state,
     json_token_number_fractional_state,
-    json_token_number_fractional2_state,
     json_token_number_exponent_state,
     json_token_number_exponent2_state,
-    json_token_number_exponent3_state,
+    json_token_number_digits_state,
     json_token_number_digitsn_state,
     json_token_number_digits2_state,
     json_token_number_digits2n_state,
@@ -224,13 +221,10 @@ static const char* json_token_number_state_get_name(
 {
     static const char* const names[] = {
         CASE_STATE(start),
-        CASE_STATE(integer),
-        CASE_STATE(integer2),
         CASE_STATE(fractional),
-        CASE_STATE(fractional2),
         CASE_STATE(exponent),
         CASE_STATE(exponent2),
-        CASE_STATE(exponent3),
+        CASE_STATE(digits),
         CASE_STATE(digitsn),
         CASE_STATE(digits2),
         CASE_STATE(digits2n),
@@ -299,6 +293,7 @@ struct json_token_string_part_t
 {
     enum json_token_string_state_t state;
     struct json_surrogate_pair_t surrogates; 
+    size_t offs;
     size_t pos;
 };
 
@@ -476,6 +471,13 @@ static void json_token_val_print_debug(
         &(part->n);              \
     })
 
+static void json_token_complete_part_print_debug(
+    const struct json_token_part_t* part)
+{
+    ASSERT(PART_TYPE_IS(complete));
+    print_debug_fmt("{}");
+}
+
 static void json_token_literal_part_print_debug(
     const struct json_token_part_t* part)
 {
@@ -504,11 +506,11 @@ static void json_token_string_part_print_debug(
         PART_VALUE(string);
 
     print_debug_fmt(
-        "{.state=%s .surrogates={.high=0x%zx .low=0x%zx} .pos=%zu}",
+        "{.state=%s .surrogates={.high=0x%zx .low=0x%zx} .offs=%zu .pos=%zu}",
             json_token_string_state_get_name(val->state),
             UINT_TO_SIZE(val->surrogates.high),
             UINT_TO_SIZE(val->surrogates.low),
-            val->pos);
+            val->offs, val->pos);
 }
 
 static void json_token_rstring_part_print_debug(
@@ -534,8 +536,8 @@ static void json_token_rstring_part_print_debug(
             val->ptr, val->pos);
 }
 
-#define CASE_PART2(n, f) [json_token_ ## n ## _part] = { .name = #n, .func = f }
-#define CASE_PART(n)     CASE_PART2(n, json_token_ ## n ## _part_print_debug)
+#define CASE_PART(n) [json_token_ ## n ## _part] = { \
+    .name = #n, .func = json_token_ ## n ## _part_print_debug }
 
 static void json_token_part_print_debug(const struct json_token_part_t* part)
 {
@@ -545,7 +547,7 @@ static void json_token_part_print_debug(const struct json_token_part_t* part)
         void (*func)(const struct json_token_part_t*);
     };
     static const struct part_info_t infos[] = {
-        CASE_PART2(complete, NULL),
+        CASE_PART(complete),
         CASE_PART(literal),
         CASE_PART(number),
         CASE_PART(string),
@@ -555,10 +557,10 @@ static void json_token_part_print_debug(const struct json_token_part_t* part)
 
     print_debug_str("{.type=");
 
-    if ((p = ARRAY_NULL_ELEM_REF(infos, part->type))) {
+    if ((p = ARRAY_NULL_ELEM_REF(infos, part->type)) &&
+        p->name && p->func) {
         print_debug_fmt("%s ", p->name);
-        if (p->func)
-            p->func(part);
+        p->func(part);
     }
     else
         print_debug_fmt("%d", part->type);
@@ -647,9 +649,7 @@ static inline void json_mem_buf_reset(struct json_mem_buf_t* buf)
 #define JSON_MEM_BUF_REMOVE(buf, n)  MEM_BUF_REMOVE(buf, n)
 
 static enum json_code_point_type_t json_mem_buf_encode_utf8(
-    struct json_mem_buf_t* buf,
-    struct json_surrogate_pair_t* surrogates,
-    size_t pos)
+    struct json_mem_buf_t* buf, struct json_token_string_part_t* part)
 {
     enum json_code_point_type_t r;
     size_t w;
@@ -658,31 +658,35 @@ static enum json_code_point_type_t json_mem_buf_encode_utf8(
     ASSERT(buf->size >= buf->len);
 
     // stev: pos is inside buf, at 4 bytes from the end
-    ASSERT(buf->len == SIZE_ADD(pos, SZ(4)));
+    ASSERT(buf->len == SIZE_ADD(part->pos, SZ(4)));
 
     STATIC(json_basic_plane_code_point == 0);
 
     if (!(r = json_encode_utf8(
-                surrogates, buf->ptr + pos, &w))) {
+                &part->surrogates, buf->ptr + part->pos, &w))) {
         ASSERT(w <= SZ(4) && w > 0);
-        buf->len -= SZ(4) - w;
+        w = SZ(4) - w;
+
+        part->offs += w;
+        buf->len -= w;
     }
 
     return r;
 }
 
 static void json_mem_buf_encode_utf8_surrogate_pair(
-    struct json_mem_buf_t* buf,
-    const struct json_surrogate_pair_t* surrogates,
-    size_t pos)
+    struct json_mem_buf_t* buf, struct json_token_string_part_t* part)
 {
     // stev: main invariant of buf
     ASSERT(buf->size >= buf->len);
 
-    // stev: pos is inside buf, at 8 bytes from the end
-    ASSERT(buf->len == SIZE_ADD(pos, SZ(8)));
+    // stev: pos is inside buf, at 4 bytes from the end
+    ASSERT(buf->len == SIZE_ADD(part->pos, SZ(4)));
 
-    json_encode_utf8_surrogate_pair(surrogates, buf->ptr + pos);
+    json_encode_utf8_surrogate_pair(
+        &part->surrogates, buf->ptr + SIZE_SUB(part->pos, SZ(4)));
+
+    part->offs += SZ(4);
     buf->len -= SZ(4);
 }
 
@@ -800,17 +804,12 @@ static bool json_parse_number_token(
         case JSON_NUM_STATE(start):
             ASSERT(muf->len == 0);
             ASSERT(ch == '-' || IS_DIGIT(ch));
-
-            part->state = JSON_NUM_STATE(integer);
-            goto next_state;
-
-        case JSON_NUM_STATE(integer):
-            part->state = JSON_NUM_STATE(integer2);
+            part->state = JSON_NUM_STATE(digits);
             if (ch != '-')
                 goto next_state;
             break;
 
-        case JSON_NUM_STATE(integer2):
+        case JSON_NUM_STATE(digits):
             if (!IS_DIGIT(ch)) {
                 part->state = JSON_NUM_STATE(error);
                 goto loop_out;
@@ -856,38 +855,22 @@ static bool json_parse_number_token(
             break;
 
         case JSON_NUM_STATE(fractional):
-            part->state = ch == '.'
-                ? JSON_NUM_STATE(fractional2)
-                : JSON_NUM_STATE(exponent);
-            goto next_state;
-
-        case JSON_NUM_STATE(fractional2):
             if (ch != '.') {
-                part->state = JSON_NUM_STATE(error);
-                goto loop_out;
+                part->state = JSON_NUM_STATE(exponent);
+                goto next_state;
             }
             part->state = JSON_NUM_STATE(digits2);
             break;
 
         case JSON_NUM_STATE(exponent):
-            if (ch == 'e' || ch == 'E') {
-                part->state = JSON_NUM_STATE(exponent2);
-                goto next_state;
-            }
-            else {
+            if (ch != 'e' && ch != 'E') {
                 done = true;
                 goto loop_out;
             }
-
-        case JSON_NUM_STATE(exponent2):
-            if (ch != 'e' && ch != 'E') {
-                part->state = JSON_NUM_STATE(error);
-                goto loop_out;
-            }
-            part->state = JSON_NUM_STATE(exponent3);
+            part->state = JSON_NUM_STATE(exponent2);
             break;
 
-        case JSON_NUM_STATE(exponent3):
+        case JSON_NUM_STATE(exponent2):
             part->state = JSON_NUM_STATE(digits3);
             if (ch != '-' && ch != '+')
                 goto next_state;
@@ -941,7 +924,6 @@ static bool json_parse_number_end_token(struct json_obj_t* obj)
         muf = val->buf;
 
         JSON_MEM_BUF_APPEND(muf, CH(0));
-
         obj->current_tok.part.type = json_token_complete_part;
         return true;
     }
@@ -1022,8 +1004,10 @@ static bool json_parse_string_token(
                 goto loop_out;
             }
             else
-            if (ch == '\\')
+            if (ch == '\\') {
                 part->state = JSON_STR_STATE(esc_char);
+                part->offs ++;
+            }
             else
             if (ch != '"')
                 JSON_MEM_BUF_APPEND(muf, ch);
@@ -1038,6 +1022,7 @@ static bool json_parse_string_token(
                     goto loop_out;
                 }
                 part->state = JSON_STR_STATE(hexa1);
+                part->offs ++;
             }
             else
             if (!IS_ASCII(ch) || (ch = unescape[ch]) == 0) {
@@ -1056,6 +1041,7 @@ static bool json_parse_string_token(
                 goto loop_out;
             }
             part->state = JSON_STR_STATE(esc_char2);
+            part->offs ++;
             break;
 
         case JSON_STR_STATE(esc_char2):
@@ -1064,6 +1050,7 @@ static bool json_parse_string_token(
                 goto loop_out;
             }
             part->state = JSON_STR_STATE(hexa5);
+            part->offs ++;
             break;
 
         case JSON_STR_STATE(hexa1) ... JSON_STR_STATE(hexa8):
@@ -1093,8 +1080,7 @@ static bool json_parse_string_token(
             else {
                 enum json_code_point_type_t t;
 
-                t = json_mem_buf_encode_utf8(
-                        muf, &part->surrogates, part->pos);
+                t = json_mem_buf_encode_utf8(muf, part);
 
                 if (part->state == JSON_STR_STATE(hexa4)) {
                     if (t == json_basic_plane_code_point)
@@ -1111,10 +1097,7 @@ static bool json_parse_string_token(
                 else { // part->state == JSON_STR_STATE(hexa8)
                     if (t == json_low_surrogate_code_point) {
                         json_mem_buf_encode_utf8_surrogate_pair(
-                            muf,
-                            &part->surrogates,
-                            SIZE_SUB(part->pos, SZ(4)));
-
+                            muf, part);
                         part->state = JSON_STR_STATE(plain_char);
                     }
                     else {
@@ -1148,6 +1131,9 @@ loop_out:
             !json_mem_buf_validate_utf8(muf, &pos)) {
             struct json_token_pos_t err = obj->current_tok.pos;
 
+            ASSERT_SIZE_ADD_NO_OVERFLOW(err.col, part->offs);
+            err.col += part->offs;
+
             ASSERT_SIZE_INC_NO_OVERFLOW(pos);
             ASSERT_SIZE_ADD_NO_OVERFLOW(err.col, pos + 1);
             err.col += pos + 1;
@@ -1451,26 +1437,24 @@ parse_out:
 
 #define JSON_PARSE_LITERAL_BUF_PART_INIT_NUMBER()
 
-#define JSON_PARSE_LITERAL_BUF_PART_INIT_STRING()         \
-    ({                                                    \
-        obj->current_tok.part.string.surrogates.high = 0; \
-        obj->current_tok.part.string.surrogates.low = 0;  \
-        obj->current_tok.part.string.pos = 0;             \
-        obj->current_tok.is_raw_str = false;              \
+#define JSON_PARSE_LITERAL_BUF_PART_INIT_STRING() \
+    ({                                            \
+        obj->current_tok.is_raw_str = false;      \
     })
 
 #define JSON_PARSE_LITERAL_BUF_PART_INIT_RSTRING() \
     ({                                             \
-        obj->current_tok.part.rstring.ptr = NULL;  \
-        obj->current_tok.part.rstring.pos = 0;     \
         obj->current_tok.is_raw_str = true;        \
     })
 
-#define JSON_PARSE_LITERAL_BUF_PART_INIT(m, p)                            \
-    ({                                                                    \
-        obj->current_tok.part.type = json_token_ ## p ## _part;           \
-        obj->current_tok.part.p.state = json_token_ ## p ## _start_state; \
-        JSON_PARSE_LITERAL_BUF_PART_INIT_ ## m();                         \
+#define JSON_PARSE_LITERAL_BUF_PART_INIT(m, p)         \
+    ({                                                 \
+        obj->current_tok.part.type =                   \
+            json_token_ ## p ## _part;                 \
+        STATIC(json_token_ ## p ## _start_state == 0); \
+        memset(&obj->current_tok.part.p,               \
+            0, sizeof(obj->current_tok.part.p));       \
+        JSON_PARSE_LITERAL_BUF_PART_INIT_ ## m();      \
     })
 
 #define JSON_PARSE_LITERAL_BUF_PART(m, n, p)                \
@@ -1954,18 +1938,24 @@ next_state:
 enum json_parse_status_t json_parse(
     struct json_obj_t* obj, const uchar_t* buf, size_t len)
 {
-    if (JSON_STATUS_IS(ok))
-        return JSON_PARSE_RETURN(json_do_parse(obj, buf, len, false));
-    else
-        return JSON_PARSE_RETURN(JSON_CURRENT_STATUS());
+    enum json_parse_status_t r;
+
+    r = JSON_STATUS_IS(ok)
+        ? json_do_parse(obj, buf, len, false)
+        : JSON_CURRENT_STATUS();
+
+    return JSON_PARSE_RETURN(r);
 }
 
 enum json_parse_status_t json_parse_done(struct json_obj_t* obj)
 {
-    if (JSON_STATUS_IS(ok))
-        return JSON_PARSE_RETURN(json_do_parse(obj, NULL, 0, true));
-    else
-        return JSON_PARSE_RETURN(JSON_CURRENT_STATUS());
+    enum json_parse_status_t r;
+
+    r = JSON_STATUS_IS(ok)
+        ? json_do_parse(obj, NULL, 0, true)
+        : JSON_CURRENT_STATUS();
+
+    return JSON_PARSE_RETURN(r);
 }
 
 struct json_obj_t* json_obj_create(
